
/**
 * Module Dependencies.
 */

var debug = require('debug')('jsg:formats');
var chance = require('chance')();

/**
 * Expose `fmt`
 */

module.exports = fmt;

/**
 * Fmt generates a value by format.
 */

function fmt(schema){
  var fn;

  if (fn = fmt[schema.format]) {
    var ret = fn(schema);
    debug('%s -> %j', schema.format, ret);
    return ret;
  } else {
    debug('skip un-implemented format %j', schema.format);
  }

  return null;
}

/**
 * Id.
 */

fmt.id = function(schema){
  return Math
    .random()
    .toString(16)
    .slice(2);
};

/**
 * UUID.
 */

fmt.uuid = function(schema){
  return chance.guid();
};

/**
 * IPv4.
 */

fmt.ipv4 = function(schema){
  return chance.ip();
};

/**
 * Latitude.
 */

fmt.latitude = function(schema){
  return chance.latitude();
};

/**
 * Longitude
 */

fmt.longitude = function(schema){
  return chance.longitude();
};

/**
 * URI.
 */

fmt.uri =
fmt.url = function(schema){
  return chance.url();
};

/**
 * Date-time.
 */

fmt['date'] =
fmt['date-time'] = function(schema){
  return chance.date().toISOString();
};

/**
 * Email.
 */

fmt.email = function(){
  return chance.email();
};