
/**
 * Expose `normalize`
 */

module.exports = normalize;

/**
 * Normalize normalizes the given schema.
 */

function normalize(s){
  s.title = s.title || '';
  s.description = s.description || '';
  s.example = s.example || '';
  s.properties = props(s.properties || {});
  s.required = s.required;
  s.anyOf = (s.anyOf || []).map(normalize);
  s.allOf = (s.allOf || []).map(normalize);
  s = s.allOf.reduce(copy, s);
  s = pick(s.anyOf).reduce(copy, s);
  delete s.definitions;
  delete s.allOf;
  delete s.anyOf;
  return s;
}

/**
 * Copy returns a function that copies
 * from `sub` schema to the parent `schema`.
 *
 * TODO: merge more stuff here? .enums etc...
 */

function copy(a, b){
  b.properties = props(b.properties);
  a.properties = merge(a.properties, b.properties);

  if (a.required || b.required) {
    var main = a.required || [];
    var sub = b.required || [];
    a.required = uniq(main.concat(sub));
  }
  
  return a;
}

/**
 * Props normalizes properties.
 */

function props(p){
  for (var k in p) p[k] = normalize(p[k]);
  return p;
}

/**
 * Merge merges two objects.
 */

function merge(a, b){
  for (var k in b) a[k] = b[k];
  return a;
}

/**
 * Uniq.
 */

function uniq(arr){
  var ret = [];

  for (var i = 0; i < arr.length; ++i) {
    if (!~ret.indexOf(arr[i])) ret.push(arr[i]);
  }

  return ret;
}

/**
 * Pick one or more random elements.
 */

function pick(arr){
  var n = Math.random() * arr.length + 1 | 0;
  return arr.slice(0, n);
}

