
/**
 * Module Dependencies.
 */

var debug = require('debug')('jsg:source');
var chance = require('chance')();
var formats = require('./formats');
var types = require('./types');

/**
 * Expose `Source`
 */

module.exports = Source;

/**
 * Initialize `Source`
 */

function Source(){
  this.formats = formats;
  this.types = types;
  this.paths = {};
}

/**
 * Map the given `path` to `fn`.
 */

Source.prototype.path = function(path, fn){
  return this.add('paths', arguments);
};

/**
 * Map the given `type` to `fn`.
 */

Source.prototype.type = function(type, fn){
  return this.add('types', arguments);
};

/**
 * Map the given `format` to `fn`.
 */

Source.prototype.format = function(format, fn){
  return this.add('formats', arguments);
};

/**
 * Add to `type` with `args`.
 */

Source.prototype.add = function(type, args){
  if (1 == args.length) {
    merge(this[type], args[0]);
  } else {
    this[type][args[0]] = args[1];
  }
  return this;
};

/**
 * Object generates an object.
 */

Source.prototype.object = function(path, schema, def){
  var ret = def || {};

  for (var k in schema.properties) {
    if (ret.hasOwnProperty(k)) continue;
    var req = schema.required || [];
    var required = !!~req.indexOf(k) || Math.random() > .5;
    if (!required) continue;
    var s = schema.properties[k];
    ret[k] = this.generate(path + '.' + k, s);
  }

  return ret;
};

/**
 * Generate a random value for the 
 * provided schema, if the func does not
 * generate a valid value an error is thrown.
 *
 * Order is path > formats > types.
 */

Source.prototype.generate = function(path, schema){
  schema.type = classify(schema);
  debug('%j', path);

  if (path.length) {
    for (var p in this.paths) {
      var m = p[0] == '.' ? p : '.' + p;
      if (m == path.slice(-m.length)) {
        var ret = this.paths[p](schema);
        ret = this.object(path, schema, ret);
        return ret;
      }
    }
  }

  if (schema.format) {
    var ret = this.formats(schema);
    if (ret) return ret;
  }

  return 'object' == schema.type
    ? this.object(path, schema)
    : this.types(schema);
};

/**
 * Merge merges a, b.
 */

function merge(a, b){
  for (var k in b) a[k] = b[k];
}

/**
 * Classify classifies a schema.
 */

function classify(s){
  if (s.type) return s.type;
  if (s.enum) return 'string';
  throw new Error('unknown type ' + JSON.stringify(s));
}
