
/**
 * Module Dependencies.
 */

var debug = require('debug')('jsg:types');
var chance = require('chance')();

/**
 * Expose `type`
 */

module.exports = type;

/**
 * Type generates any type.
 */

function type(schema){
  var fn;

  if (schema.enum) {
    var n = Math.random() * schema.enum.length;
    return schema.enum[n | 0];
  }

  if (fn = type[schema.type]) {
    var ret = fn(schema);
    debug('%s %j', schema.type, ret);
    return ret;
  }

  throw new Error('jsg: unknown type ' + schema.type + ' ' + JSON.stringify(schema));
}

/**
 * Boolean.
 */

type.boolean = function(schema){
  return chance.bool();
};

/**
 * Integer.
 */

type.integer = function(schema){
  return chance.integer();
};

/**
 * Number.
 */

type.number = function(schema){
  return Math.random() >= .5
    ? chance.floating()
    : chance.integer();
};

/**
 * Null.
 */

type.null = function(schema){
  return null;
};

/**
 * String.
 */

type.string = function(schema){
  var opts = schema.enum || [chance.word()];
  var i = Math.random() * opts.length;
  return opts[Math.floor(i)];
};
